/*  cssed (c) Iago Rubio 2003, 2004 - A tiny CSS editor.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define PLAT_GTK 1

#include <libgen.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#ifdef WITH_TERMINAL
#  include <vte/vte.h>
#endif

#include "cssedwindow.h"
#include "document.h"
#include "callbacks.h"
#include "selectorparser.h"
#include "cssdialogs-interface.h"
#include "configdlg-interface.h"
#include "interface.h"
#include "selector-wizard.h"
#include "color-wizard.h"
#include "box-wizard.h"
#include "cssparser.h"
#include "doc-scanner.h"
#include "support.h"
#include "utils.h"
#include "debug.h"

#include <Scintilla.h>
#include <SciLexer.h>
#include <ScintillaWidget.h>
#include <libcroco.h>

#ifdef WITH_TERMINAL
void
on_vte_terminal_child_exited            (VteTerminal *vteterminal,
                                            gpointer user_data)
{
	gchar* shell;
	
	shell = getenv("SHELL");
	if( shell == NULL){
		vte_terminal_reset (vteterminal, TRUE, TRUE);
		vte_terminal_fork_command (vteterminal, "/bin/sh", NULL, NULL, "~/",
				   0, 0, 0);
	}else{
		if( g_file_test(shell, G_FILE_TEST_EXISTS) && g_file_test(shell,G_FILE_TEST_IS_EXECUTABLE) ){
			vte_terminal_reset (vteterminal, TRUE, TRUE);
			vte_terminal_fork_command (vteterminal, shell, NULL, NULL, "~/",
				   0, 0, 0);
		}else{
			vte_terminal_reset (vteterminal, TRUE, TRUE);
			vte_terminal_fork_command (vteterminal, "/bin/sh", NULL, NULL, "~/",
				   0, 0, 0);
		}
	}
}
#endif
void
on_notebook_docs_switch_page (GtkNotebook * notebook, GtkNotebookPage * page,
			      guint page_num, gpointer user_data)
{
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	GtkWidget *text_output;
	GtkTextBuffer *buffer;
	Tcssed_dyn_menus *menus;
	GtkWidget *window_widget;
	gchar* strout;

	window = CSSED_WINDOW (user_data);
	menus = window->dynamic_menus;

	DBGMSG ("callbacks.c: on_notebook_docs_switch_page - Page switched to %d, notebook %x, page %x", page_num, notebook, page);

	tmpdoc = document_get_from_notebook_page_num (window, page_num);
	if (tmpdoc == NULL)
	{
		cssed_error_message
			(_("Programming error !!"),
			 _("An invalid document has been passed,\nthings are going to fail from this point.\nPlease tell the developers this has occurred."));
		return;
	}
	else
	{
		gtk_widget_grab_focus (GTK_WIDGET (tmpdoc->editor));
		gtk_widget_queue_resize (GTK_WIDGET (tmpdoc->editor));
#ifdef DEBUG
		if (tmpdoc->filename != NULL)
			DBGMSG ("callbacks.c: on_notebook_docs_switch_page  - filename %s", tmpdoc->filename);
		else
			DBGMSG ("callbacks.c: on_notebook_docs_switch_page() - filename NULL !!!!");
#endif
		window_widget =
			lookup_widget (window->notebook_docs, "window");
		strout = g_strdup_printf("cssed: %s",tmpdoc->filename ==
				      NULL ? _("Untitled doc") : tmpdoc->filename );
		
		gtk_window_set_title (GTK_WINDOW (window_widget),strout);
		g_free(strout);
	}
	
}
void
on_menu_doc_info_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	GtkListStore *store;
	GtkTreeIter iter;
	gint len;
	gchar* lenmsg;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	store = window->output_store;

	gtk_list_store_clear (store);
	gtk_list_store_insert (store, &iter, 0);

	if (tmpdoc->filename == NULL)
	{
		gtk_list_store_set (store, &iter, 0, _("Untitled doc"), -1);
	}
	else
	{
		gtk_list_store_set (store, &iter, 0, tmpdoc->filename, -1);
	}

	gtk_list_store_insert (store, &iter, 1);

	if (tmpdoc->modified)
	{
		gtk_list_store_set (store, &iter, 0, _("Document modified (not saved)"), -1);
	}
	else
	{
		gtk_list_store_set (store, &iter, 0, _("Document not modified (saved)"),
				    -1);
	}
	
	gtk_list_store_insert (store, &iter, 2);
	len = document_get_length(tmpdoc);
	lenmsg = g_strdup_printf(_("Document length: %d"),len); 
	gtk_list_store_set (store, &iter, 0, lenmsg,   -1);
	g_free(lenmsg);
	
	gtk_list_store_insert (store, &iter, 3);
	
	if( tmpdoc->autocompletion ){
		gtk_list_store_set (store, &iter, 0, _("Autocompletion enabled for this document"),   -1);
	}else{
		gtk_list_store_set (store, &iter, 0, _("Autocompletion disabled for this document"),   -1);
	}	
	if( document_get_folding_margin_visible(tmpdoc) ){
		gtk_list_store_set (store, &iter, 0, _("Folding enabled for this document"),   -1);
	}else{
		gtk_list_store_set (store, &iter, 0, _("Folding disabled for this document"),   -1);
	}	
}

/* validation and dump*/
void
on_menu_utils_validate_activate(GtkMenuItem * menuitem, gpointer user_data)
{

	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gint doclen;
	gchar* doc_buffer;
	gchar* endbuff;
	
	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	
	doclen = document_get_length (tmpdoc);
	doc_buffer = g_malloc0(++doclen);
	document_get_text (tmpdoc, doclen, doc_buffer);
	endbuff = g_strconcat(doc_buffer, "\n",NULL);	
	document_clear_validator_arrow( tmpdoc );
	cssed_cr_parser_parse_buffer (tmpdoc, endbuff,strlen(endbuff),TRUE);
	g_free( endbuff);
	g_free(doc_buffer);
}
/* only validation */
void 
on_menu_utils_validate_only_activate	(GtkMenuItem 	  * menuitem,
								gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gint doclen;
	gchar* doc_buffer;
	gchar* endbuff;
	
	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	
	
	doclen = document_get_length (tmpdoc);
	doc_buffer = g_malloc (++doclen);
	document_get_text (tmpdoc, doclen, doc_buffer);
	endbuff = g_strconcat(doc_buffer, "\n",NULL);	
	document_clear_validator_arrow( tmpdoc );
	cssed_cr_parser_parse_buffer (tmpdoc, endbuff,strlen(endbuff),FALSE);
	g_free( endbuff);
	g_free(doc_buffer);	
}


/* end validation */

/* this cleans the otuput list store */
void 
on_menu_utils_cleanoutput_activate		
				(GtkMenuItem 	  * menuitem,       gpointer user_data)
{
	GtkListStore *output_store;
	
	output_store = GTK_LIST_STORE(user_data);
	gtk_list_store_clear( output_store );	
}

void
on_menu_new_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkNotebook *notebook;
	GtkWidget *editor;
	GtkWidget *label;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);
	notebook = GTK_NOTEBOOK (window->notebook_docs);

	if (!GTK_IS_NOTEBOOK (notebook))
	{
		cssed_error_message
			(_("Programming error"),_("Please see standard output and tell\n\nthe developers what happened !!!"));
		DBGMSG ("callbacks.c:  on_menu_new_activate()- %x Is NOT a GtkNotebook", notebook);
		return;
	}
	create_and_attach_new_doc (window, _("Untitled"));
}
void
on_menu_abrir_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	GtkWidget *opensel;
	
	window = CSSED_WINDOW (user_data);
	opensel = create_open_fileselection (window);
	if (opensel != NULL)
	{
		gtk_widget_show (opensel);
	}
}
void
on_menu_guardar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	document_save( tmpdoc );
}
void
on_save_fileselection_ok_button_clicked (GtkButton * button,
					 gpointer user_data)
{
	Tcssed_doc *savedoc;
	GtkWidget *notebook;
	GtkWidget *file_selector;
	Tcssed_fileselector_data *data;
	Tcssed_window *window;	// main window object
	gchar *doc_buffer;
	gint doclen;
	size_t nwriten;
	const gchar *selected_filename;
	FILE *f;
	gchar *msg;
	
	data = (Tcssed_fileselector_data *) user_data;
	window = (Tcssed_window *) data->main_window;
	notebook = GTK_WIDGET (data->notebook);
	file_selector = GTK_WIDGET (data->fileselector);
	savedoc = document_get_current (window);

	selected_filename =
		gtk_file_selection_get_filename (GTK_FILE_SELECTION
						 (file_selector));
	gtk_widget_destroy (gtk_widget_get_toplevel (file_selector));	// destroy the selector

	f = fopen (selected_filename, "w");
	if (f == NULL)
	{
		msg = g_strdup_printf ("The file %s\ncannot be opened for writting.",
				       selected_filename);
		cssed_error_message ("Can't open file", msg);
		g_free (msg);
	}
	else
	{
		doclen = document_get_length (savedoc);
		doc_buffer = g_malloc (++doclen);
		document_get_text (savedoc, doclen, doc_buffer);

		nwriten = fwrite (doc_buffer, sizeof (gchar), doclen, f);
		if (nwriten != doclen)
		{
				cssed_error_message
					("Error while writing to file",
					 "There is a mismatch between the file size\nand the bytes written.\nThe file may be damaged.");
		}
		document_set_modified (savedoc, FALSE);
		document_empty_undo_buffer(savedoc);
		g_free (doc_buffer);
		fclose (f);
	}

	g_free (data);
}
void
on_save_as_fileselection_ok_button_clicked (GtkButton * button,
					    gpointer user_data)
{
	Tcssed_doc *savedoc;
	Tcssed_fileselector_data *data;
	Tcssed_window *window;	// main window object
	GtkWidget *window_widget;
	GtkWidget *notebook;
	GtkWidget *file_selector;
	GtkWidget *editor;
	GtkWidget* new_label;
	const gchar *selected_filename;
	gchar *doc_buffer;
	gchar *filename;
	gint doclen;
	gint current_index;
	size_t nwriten;
	FILE *f;
	gchar *msg;
	gchar *test;

	data = (Tcssed_fileselector_data *) user_data;
	window = (Tcssed_window *) data->main_window;
	notebook = GTK_WIDGET (data->notebook);
	file_selector = GTK_WIDGET (data->fileselector);
	savedoc = document_get_current (window);
	window_widget = lookup_widget(notebook,"window");
	
	selected_filename =
		gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));
	gtk_widget_destroy (gtk_widget_get_toplevel (file_selector));	// destroy the selector

	f = fopen (selected_filename, "w");
	if (f == NULL)
	{
		msg = g_strdup_printf (_("The file %s\ncannot be opened for writting."),
				       selected_filename);
		cssed_error_message (msg,_("Can't open file"));
		g_free (msg);
	}
	else
	{
		doclen = document_get_length (savedoc);
		doc_buffer = g_malloc (++doclen);
		document_get_text (savedoc, doclen, doc_buffer);

		nwriten = fwrite (doc_buffer, sizeof (gchar), doclen, f);
		if (nwriten != doclen)
		{
			cssed_error_message
			(
			_("There is a mismatch between the file size\nand the bytes written.\nThe file may be damaged."),
			_("Error while writing to file")
			);
		}
		document_empty_undo_buffer(savedoc);
		document_set_modified (savedoc, FALSE);
		current_index =
			gtk_notebook_get_current_page (GTK_NOTEBOOK
						       (notebook));
		editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
						    current_index);
		filename = g_strdup (selected_filename);
		new_label = gtk_notebook_get_tab_label(GTK_NOTEBOOK (notebook),
                                         editor);
		if( new_label != NULL ){
			gtk_label_set_text(GTK_LABEL(new_label), basename(filename));
		}

		g_free (savedoc->filename);
		savedoc->filename = filename;
		
		gtk_window_set_title(GTK_WINDOW(window_widget),filename);
		g_free (doc_buffer);
		fclose (f);
	}
	g_free (data);
}
void
on_menu_guardar_como_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *file_selector;
	Tcssed_window *window;	// main window object

	window = CSSED_WINDOW (user_data);
	file_selector = create_save_as_fileselection (window);
	gtk_widget_show (file_selector);
}
// called on window close
void
on_menu_salir_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;	// main window object
	Tcssed_doc* tmpdoc;
	GtkWidget* window_widget;
	GList* doclist;
	GList* unsaved_docs = NULL;
	gint ntosave;
	GtkWidget* dlg;
	gint response;
	gint untitled_count = 0;
	gchar* name;
	
	window = CSSED_WINDOW (user_data);
	doclist = g_list_first((GList*)  window->document_list);
	window_widget = lookup_widget(window->notebook_docs,"window");
	while( doclist != NULL ){
		tmpdoc = (Tcssed_doc*) doclist->data;
		if( document_get_modified(tmpdoc) ){
			unsaved_docs = g_list_append(unsaved_docs, tmpdoc);
		}
		doclist = g_list_next(doclist);
	}
	ntosave = g_list_length(unsaved_docs);
	if( ntosave > 0 ){
		dlg = create_unsaved_files_dialog();
		response = gtk_dialog_run( GTK_DIALOG(dlg) );
		switch( response ){
			case GTK_RESPONSE_OK: // save all avoiding unnamed
				while( unsaved_docs != NULL ){
					tmpdoc = (Tcssed_doc*) unsaved_docs->data;
					if( tmpdoc->filename != NULL ){
						document_save( (Tcssed_doc*) tmpdoc );
					}else{
						name = g_strdup_printf(_("untitled_%d"),untitled_count);
						tmpdoc->filename = name;
						document_save( (Tcssed_doc*) tmpdoc );
						untitled_count++;
						g_free( name );
					}
					unsaved_docs = g_list_next(unsaved_docs);
				}
				break;
			case GTK_RESPONSE_ACCEPT: // choose files
				while( unsaved_docs != NULL ){
					GtkWidget* confirm;
					gchar* msg;
									
					tmpdoc = (Tcssed_doc*) unsaved_docs->data;
					if( tmpdoc->filename == NULL ){
						name = g_strdup_printf(_("untitled_%d"),untitled_count);
						tmpdoc->filename = name;
						untitled_count++;
					}					
					msg = g_strdup_printf(_("File name: %s"), tmpdoc->filename);
					confirm = create_ok_cancel_dialog(_("Do you want to save this file?"),msg);
					response = gtk_dialog_run( GTK_DIALOG(confirm) );
					if( response == GTK_RESPONSE_OK ){
						document_save(tmpdoc);
					}
					gtk_widget_destroy( GTK_WIDGET( confirm ) );
					if( name ) g_free( name);
					g_free( msg );
					msg = NULL;
					name = NULL;					
					unsaved_docs = g_list_next(unsaved_docs);
 				}
				break;
			case GTK_RESPONSE_CANCEL: // canceled by user don't quit
				gtk_widget_destroy( dlg );
				return;
				break;
		}		
	}
	save_config_to_file(window);
	gtk_main_quit ();
}

void
on_menu_undo_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	on_button_undo_clicked( NULL, user_data);
}
	
void
on_menu_redo_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	on_button_redo_clicked( NULL, user_data );
}
	
void
on_menu_search_activate (GtkMenuItem *menuitem, gpointer  user_data)
{
	on_button_buscar_clicked( NULL, user_data );
}
	
void
on_menu_replace_activate (GtkMenuItem  *menuitem, gpointer user_data)
{
	on_button_reemplazar_clicked( NULL, user_data );
}

void
on_menu_cortar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_button_cortar_clicked (NULL, user_data);
}
void
on_menu_copiar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_button_copiar_clicked (NULL, user_data);
}
void
on_menu_pegar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_button_pegar_clicked (NULL, user_data);
}


void
on_menu_borrar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	
	document_clear_sel(tmpdoc);
}

void
on_menu_preferences_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	on_button_preferences_clicked( NULL, user_data );
}

void
on_menu_view_toolbar_toggled(GtkCheckMenuItem      *checkmenuitem,   
	gpointer user_data)
{
	gboolean active;	
	active =  gtk_check_menu_item_get_active( checkmenuitem );
	if( active ){
		gtk_widget_show(GTK_WIDGET(user_data));
	}else{
		gtk_widget_hide(GTK_WIDGET(user_data));
	}
}

void
on_menu_about_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *about = create_about ();
	if (about != NULL)
		gtk_widget_show (about);
}

/*    For future use
void 
on_menu_cssed_help_activate  (GtkMenuItem  *menuitem, gpointer  user_data)
{   // THIS IS OLD TESTING STUFF
	GtkWidget *help_window = create_help_window ();
	if (help_window != NULL){
		gtk_widget_show (help_window);
		help_window_show_file (help_window,"t1.html");
	}
}
*/

/* dynamic menu items */       
void
on_menu_line_numbers_toggled (GtkCheckMenuItem * checkmenuitem,
			      gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_line_numbers_toggled  - document NULL");
		return;
	}
	else
	{
		visible = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_line_numbers_visible (tmpdoc, visible);
	}
}

void
on_menu_line_endings_toggled (GtkCheckMenuItem * checkmenuitem,
			      gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_line_endings_toggled - document NULL");
		return;
	}
	else
	{
		visible = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_line_endings_visible (tmpdoc, visible);
	}
}

void
on_menu_line_wrapping_toggled (GtkCheckMenuItem * checkmenuitem,
			       gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean wrapped;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_line_endings_toggled - document NULL");
		return;
	}
	else
	{
		wrapped = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_lines_wrapped (tmpdoc, wrapped);
	}
}

void
on_menu_ver_white_spaces_toggled (GtkCheckMenuItem * checkmenuitem,
				  gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{	// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_ver_white_spaces_toggled - document NULL");
		return;
	}
	else
	{
		visible = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_white_spaces_visible (tmpdoc, visible);
	}
}

void
on_menu_item_autocompletion_toggled	(GtkCheckMenuItem      *checkmenuitem,   
	gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean active;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);	
	
	active =  gtk_check_menu_item_get_active (checkmenuitem);
	tmpdoc->autocompletion = active;
}

void
on_menu_item_folding_toggled	(GtkCheckMenuItem      *checkmenuitem,   
	gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean fold;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_line_numbers_toggled  - document NULL");
		return;
	}
	else
	{
		fold = gtk_check_menu_item_get_active (checkmenuitem);
		if( !fold )
			document_unfold_all(tmpdoc);	
		document_set_folding_margin_visible (tmpdoc, fold);
	}
}

void
on_menu_item_font_activate( GtkMenuItem *menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	GtkWidget* font_selector;
	gint response;
	gchar* font;
	gchar* family_name;
	gint size;
	PangoFontDescription* pfd; 
	
	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);		
	
	font_selector = gtk_font_selection_dialog_new   (_("Select font for current document") );
	
	response = gtk_dialog_run(GTK_DIALOG(font_selector));
	if( response == GTK_RESPONSE_OK ){
		font =  gtk_font_selection_dialog_get_font_name(
							GTK_FONT_SELECTION_DIALOG(font_selector));
		if( font != NULL){
			pfd = pango_font_description_from_string( font );
			family_name = (gchar*) pango_font_description_get_family ( pfd );
			size = pango_font_description_get_size ( pfd );
			font = g_strdup_printf("!%s",family_name);
			document_set_font_by_name(tmpdoc, font);
			document_set_font_size(tmpdoc, size/PANGO_SCALE);
			document_colourise_all(tmpdoc);	
			g_free(font);
		}
	}
	gtk_widget_destroy(font_selector);
}

void 
on_menuitem_folding_unfoldall_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);	
	document_unfold_all(tmpdoc);
}
void
on_menuitem_fonding_foldall_activate(GtkMenuItem  *menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_fold_all(tmpdoc);	
}
void 
on_menuitem_highlighting_clear_activate(GtkMenuItem *menuitem,  
										gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);	
	document_style_clear_all(tmpdoc);
	document_set_font(tmpdoc);
	document_set_current_font_size(tmpdoc);
}
void
on_menuitem_highlighting_apply_activate(GtkMenuItem *menuitem, 
										gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);	
	document_set_css_style(tmpdoc);
	document_set_font(tmpdoc);
	document_set_current_font_size(tmpdoc);
	document_colourise_all(tmpdoc);	
}
void
on_menuitem_selector_wizard_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc* doc;
	GtkWidget* selector_wizard;
	GtkWidget* notebook;
	GtkWidget* treeview;
	GtkWidget* radio;
	GtkTreeModel* model;
	GtkTreeIter iter;
	gboolean one_declaration_for_each;
	gboolean valid;
	gchar *strtoadd;
	gchar *strout;	
	gchar* list_name;
	gint page_active;
	gint response;
	

	window = CSSED_WINDOW (user_data);	
	selector_wizard = create_selector_wizard ();
	response = gtk_dialog_run(GTK_DIALOG(selector_wizard));
	doc = document_get_current(window);
	
	if( response ==  GTK_RESPONSE_OK)
	{
		notebook = lookup_widget(selector_wizard, "notebook_wizard");
		radio = lookup_widget(selector_wizard, "radiobutton_for_each");
		page_active = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
		switch( page_active ){
			case 0: // selector
				list_name = "treeview_selector_selected";
				break;
			case 1: // child
				list_name = "treeview_child_selected";
				break;				
			case 2: //class
				list_name = "treeview_class_selected";
				break;
			case 3: // sibling
				list_name = "treeview_sibling_selected";
				break;
			case 4: // contextual
				list_name = "treeview_contextual_selected";
				break;	
			default:
				DBGMSG("on_menuitem_selector_wizard_activate: notebook page out of range");
				return;
		}
		treeview = lookup_widget(selector_wizard,list_name);
		model = gtk_tree_view_get_model (GTK_TREE_VIEW(treeview));
		one_declaration_for_each =
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio));			
		valid = gtk_tree_model_get_iter_first(model, &iter);
		
		if( one_declaration_for_each ){
			while(valid){
				gtk_tree_model_get (model, &iter, 0, &strtoadd, -1);
				strout = g_strdup_printf("%s {\n\n}\n\n",strtoadd);
				document_add_text(doc,strout);
				g_free(strout);
				valid = gtk_tree_model_iter_next(model,&iter);
			}				
		}else{
			while(valid){
				gtk_tree_model_get (model, &iter, 0, &strtoadd, -1);
				strout = g_strdup_printf("%s",strtoadd);
				document_add_text(doc,strout);
				g_free(strout);
				valid = gtk_tree_model_iter_next(model,&iter);
				if(valid) document_add_text(doc,",\n");					
			}	
			document_add_text(doc,"	{\n\n}\n\n");
		}
	}
	gtk_widget_destroy(selector_wizard);
}

void
on_menuitem_box_wizard_activate	(GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget* dlg;
	dlg = create_box_wizard (CSSED_WINDOW(user_data));
	gtk_widget_show(dlg);
}

void
on_menuitem_color_wizard_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	GtkWidget* color_wizard;
	Tcssed_window *window;
	
	window = CSSED_WINDOW (user_data);
	color_wizard = create_color_wizard_dialog (window);	
	gtk_widget_show(color_wizard);
}

void
on_menu_eol_mode_cr_activate (GtkMenuItem * checkmenuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	
	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_eol_mode_cr_activate - document NULL");
		return;
	}
	else
	{
		document_convert_eols (tmpdoc, SC_EOL_CR);
	}
}
// needs fix
void
on_menu_eol_mode_lf_activate (GtkMenuItem * checkmenuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_eol_mode_lf_activate - document NULL");
		return;
	}
	else
	{
		document_convert_eols (tmpdoc, SC_EOL_LF);
	}
}


// needs fix
void
on_menu_eol_mode_crlf_activate (GtkMenuItem * checkmenuitem,
				gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		DBGMSG ("callbacks.c:on_menu_eol_mode_crlf_activate - document NULL");
		return;
	}
	else
	{
		document_convert_eols (tmpdoc, SC_EOL_CRLF);
	}
}

/* set the child menus state */
void
on_menu_view_activate (GtkMenuItem * checkmenuitem, gpointer user_data)
{
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	Tcssed_dyn_menus *menus;
	gboolean visible;
	gint eolmode;

	window = CSSED_WINDOW (user_data);
	menus = window->dynamic_menus;
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{
		cssed_error_message
			(
			_("Can't get current document to change menus.\nMenus state will be out of sync with the document\n"),
			_("Menu error")
			);
		return;
	}
	visible = document_get_line_numbers_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_line_numbers, visible);
	visible = document_get_line_endings_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_endings, visible);
	visible = document_get_lines_wrapped (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_lines_wrapped, visible);
	visible = document_get_white_spaces_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_white_spaces, visible);
	if( tmpdoc->autocompletion ){
		gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_autocompletion, TRUE);
	}else{
		gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_autocompletion, FALSE);
	}
	// ADD FOLDING HERE
	visible = document_get_folding_margin_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_folding, visible );

}

/* 
	scan the current selector: extract the buffer and calls 
	parser_css_selector_buffer_to_list()
*/
void
on_menu_scan_selector_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	ScintillaObject *sci;
	gint position;
	gint open_brace;
	gint close_brace;
	gchar *buffer;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	sci = (ScintillaObject *) sci_get_from_document (tmpdoc);
	position = document_get_current_position (tmpdoc);

	DBGMSG ("callbacks.c:on_menu_scan_selector_activate() Position at %d", position);

	// set the anchor
	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	open_brace = SSM (sci, SCI_SEARCHPREV, 0, (sptr_t) "{");

#ifdef DEBUG
	if (open_brace == -1)
		DBGMSG ("callbacks.c:on_menu_scan_selector_activate() No open brace before");
	else
		DBGMSG ("callbacks.c:on_menu_scan_selector_activate() Open brace at %d", open_brace);
#endif

	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	close_brace = SSM (sci, SCI_SEARCHNEXT, 0, (sptr_t) "}");

#ifdef DEBUG
	if (close_brace == -1)
		DBGMSG ("callbacks.c:on_menu_scan_selector_activate() No close brace after");
	else
		DBGMSG ("callbacks.c:on_menu_scan_selector_activate() Close brace at %d", close_brace);
#endif

	if (close_brace < position)
	{
		cssed_error_message
			(
			_("You should place the cursor between the opening\nand closing breaces of any declaration to be scanned\n"),
			_("The cursor must be into a declaration.")
			);
	}
	else
	{
		if ((open_brace != -1) && (close_brace != -1))
		{		// we're in a selector then
			SSM (sci, SCI_SETSEL, open_brace + 1, close_brace);
			buffer = g_malloc ((close_brace -
					    open_brace) * sizeof (gchar));
			SSM (sci, SCI_GETSELTEXT, 0, (sptr_t) buffer);

			g_strstrip (buffer);
			parser_css_selector_buffer_to_list (buffer, window,
							    tmpdoc);
		}
	}
}

void
on_show_footer_panel_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(menuitem),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));

	gtk_paned_set_position(GTK_PANED(user_data) ,height-(height/3));
}

void
on_hide_footer_panel_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(menuitem),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));
	gtk_paned_set_position(GTK_PANED(user_data) ,height);
}

void
on_show_side_panel_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(menuitem),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));

	gtk_paned_set_position(GTK_PANED(user_data) ,width-(width/3));
}

void
on_hide_side_panel_activate(GtkMenuItem *menuitem, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(menuitem),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));
	gtk_paned_set_position(GTK_PANED(user_data) ,width);	
}

void
on_button_abrir_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_abrir_activate (NULL, user_data);
}
void
on_button_nuevo_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_new_activate (NULL, user_data);
}
void
on_button_cerrar_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *editor;
	GtkWidget *notebook;
	GtkWidget* okcancel;
	gint response;
	gint page_num;
	Tcssed_doc *doc;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	notebook = GTK_WIDGET (window->notebook_docs);
	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
					    page_num);
	doc = (Tcssed_doc *) document_get_from_editor (window, editor);

	if (doc->modified)
	{
		okcancel = create_ok_cancel_dialog(
				_("File is not saved, Do you want to close it?"),
				_("The file has changed and the changes will be lost if you close it. Click the  <b>\"Cancel\"</b> button if you want to skip to close the file, click the <b>\"Ok\"</b> button if you want to close the file unsaved.")
				);
		 response = gtk_dialog_run( GTK_DIALOG(okcancel) );
		if( response == GTK_RESPONSE_OK ){
			document_dispose (doc);
		}	
		gtk_widget_destroy( GTK_WIDGET(okcancel) );		
	}else{
		document_dispose (doc);
	}
}
void
on_button_buscar_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *searchdlg;

	searchdlg = create_search_dialog (user_data);
	gtk_widget_show (searchdlg);
}
void
on_button_reemplazar_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *replacedlg;

	replacedlg = create_replace_dialog (user_data);	
	gtk_widget_show (replacedlg);
}
void
on_button_copiar_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_copy (tmpdoc);
}
void
on_button_pegar_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_paste (tmpdoc);
}

void
on_button_cortar_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_cut (tmpdoc);
}

void
on_button_color_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *colorselection_dlg;
	GdkColor color;
	gchar* str;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	str = g_strdup("Choose color"); // will be disposed by callbacks
	colorselection_dlg = create_color_selector_dialog (window, str, FALSE, TRUE);
	gtk_widget_show (colorselection_dlg);
}


void
on_button_guardar_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_guardar_activate (NULL, user_data);
}


void
on_button_guardarcomo_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_guardar_como_activate (NULL, user_data);
}

void
on_button_next_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;
	gint line;
	gint position;


	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	position = document_get_current_position (doc);
	line = document_get_line_from_position (doc, position);

	if (!document_marker_next (doc, ++line, 1))
	{
		cssed_error_message (
				_("There is no marker between the current\nposition and the end of the document."),
				_("No next marker")
			);
	}else{
		document_ensure_line_is_visible(doc,document_get_line_from_position(doc, document_get_current_position(doc)));
	}
}

void
on_button_prev_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;
	gint line;
	gint position;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	position = document_get_current_position (doc);
	line = document_get_line_from_position (doc, position);

	if (!document_marker_prev (doc, --line, 1))
	{
		cssed_error_message (
			_("There is no marker between the current\nposition and the start of the document."),
			_("No previous marker")
		);
	}{
		document_ensure_line_is_visible(doc,document_get_line_from_position(doc, document_get_current_position(doc)));
	}
}

void
on_button_undo_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *editor;
	GtkWidget *notebook;
	Tcssed_doc *doc;
	gint page_num;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	notebook = GTK_WIDGET (window->notebook_docs);
	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
					    page_num);
	doc = (Tcssed_doc *) document_get_from_editor (window, editor);

	document_undo (doc);
}

void
on_button_redo_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *editor;
	GtkWidget *notebook;
	Tcssed_doc *doc;
	gint page_num;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	notebook = GTK_WIDGET (window->notebook_docs);
	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
					    page_num);
	doc = (Tcssed_doc *) document_get_from_editor (window, editor);

	document_redo (doc);
}

void
on_button_zoom_in_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	document_zoom_in (doc);
}
void 
on_button_hide_side_clicked (GtkButton * button, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(button),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));
	gtk_paned_set_position(GTK_PANED(user_data) ,width);	
}
void
on_button_show_side_clicked (GtkButton * button, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(button),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));

	gtk_paned_set_position(GTK_PANED(user_data) ,width-(width/3));
}
void 
on_button_hide_footer_clicked 	(GtkButton * button, gpointer user_data)
{

	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(button),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));
	gtk_paned_set_position(GTK_PANED(user_data) ,height);

}
void 
on_button_show_footer_clicked (GtkButton * button, gpointer user_data)
{
	gint pos;
	GtkWidget* win;
	gint width;
	gint height;

	win = lookup_widget(GTK_WIDGET(button),"window");
 	gtk_window_get_size(GTK_WINDOW(win), &width, &height);
	pos = gtk_paned_get_position(GTK_PANED(user_data));

	gtk_paned_set_position(GTK_PANED(user_data) ,height-(height/3));
}
void
on_button_zoom_out_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	document_zoom_out (doc);
}

void
on_button_zoom_off_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	document_zoom_off (doc);
}

void
on_button_preferences_clicked(GtkButton * button, gpointer user_data)
{
	GtkWidget* dialog;
	gint response;
	GList* doclist;
	Tcssed_cfg* tmpcfg;
	Tcssed_cfg* cfg;
	Tcssed_window* window;
	// here will be copied the old configuration
	// to be restored if the user click on cancel
	// as changes are made directly to the window->config structure
	tmpcfg = g_malloc(sizeof(Tcssed_cfg));
	window = CSSED_WINDOW( user_data);
	cfg = (Tcssed_cfg*) window->config;
	
	dialog = create_preferences_dialog ( window, tmpcfg );

	response = gtk_dialog_run( GTK_DIALOG(dialog));
	switch( response ){
		case GTK_RESPONSE_OK:
			if( GTK_IS_WIDGET( dialog ) )
				gtk_widget_destroy( dialog );
			// colourise all			
			doclist = g_list_first( (GList*) window->document_list );	
			if( doclist != NULL ){
				while( doclist != NULL ){
					//DBGMSG("Change style in doc %p\n", doclist->data);
					//document_set_default_configuration( (Tcssed_doc*) doclist->data );
					document_set_css_style( (Tcssed_doc*) doclist->data );
					document_set_font	  ( (Tcssed_doc*) doclist->data );
					document_set_current_font_size( (Tcssed_doc*) doclist->data );
					document_colourise_all( (Tcssed_doc*) doclist->data );					
					doclist = g_list_next( doclist );
				}
			}
			break;
		default:
			// restore old configuration
			if( GTK_IS_WIDGET( dialog ) )
				gtk_widget_destroy( dialog );
			cssed_window_config_copy_style_and_defaults( tmpcfg ,
								window->config );
			break;	
	}
	g_free( tmpcfg );
}

void
on_window_destroy (GtkObject * object, gpointer user_data)
{
	gtk_main_quit();
}
gboolean    on_window_delete_event      	(GtkWidget *widget,
                                            GdkEvent *event,
                                            gpointer user_data)
{
	on_menu_salir_activate (NULL, user_data);
	return TRUE;
}
// secondary toolbar callbacks
void
on_button_border_all_clicked (GtkButton  *button, 	gpointer user_data)
{
	GtkWidget* dialog;
	dialog = create_border_properties_dialog (CSSED_WINDOW(user_data),
				 								g_strdup("border"),
												TRUE,
												TRUE);
	gtk_widget_show(dialog);	
}
void 
on_button_margin_all_clicked (GtkButton  *button, 	gpointer user_data)
{
	GtkWidget* dialog;
	dialog = create_margin_property_dialog (CSSED_WINDOW(user_data),
				 								g_strdup("margin"),
												TRUE,
												TRUE);
	gtk_widget_show(dialog);		
}
void 
on_button_padding_all_clicked (GtkButton  *button, 	gpointer user_data)
{
	GtkWidget* dialog;
	dialog = create_padding_property_dialog (CSSED_WINDOW(user_data),
				 								g_strdup("padding"),
												TRUE,
												TRUE);
	gtk_widget_show(dialog);		
}
void
on_button_box_properties_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget* dialog;
	dialog = create_box_wizard(CSSED_WINDOW(user_data));
	gtk_widget_show(dialog);
}
void 
on_button_sel_wizard_clicked (GtkButton  *button, 	gpointer user_data)
{
	on_menuitem_selector_wizard_activate(NULL, CSSED_WINDOW(user_data));	
}
void 
on_button_col_wizard_clicked (GtkButton  *button, 	gpointer user_data)
{
	GtkWidget* dialog;
	dialog = create_color_wizard_dialog(CSSED_WINDOW(user_data));
	gtk_widget_show(dialog);	
}
void 
on_button_font_clicked	 (GtkButton  *button, 	gpointer user_data)
{
	GtkWidget* dialog;
	dialog = create_font_property_dialog (CSSED_WINDOW(user_data),
				 								g_strdup("font"),
												TRUE,
												TRUE);
	gtk_widget_show(dialog);	
}
void 
on_button_scan_selector_clicked  (GtkButton  *button, 	gpointer user_data)
{
	on_menu_scan_selector_activate(NULL, user_data);
}
void 
on_button_doc_info_clicked (GtkButton  *button, 	gpointer user_data)
{
	on_menu_doc_info_activate(NULL, user_data);
}
void 
on_button_clean_output_clicked  (GtkButton  *button, 	gpointer user_data)
{
	on_menu_utils_cleanoutput_activate(NULL, user_data);
}
void 
on_button_validate_clicked  (GtkButton  *button, 	gpointer user_data)
{
	on_menu_utils_validate_only_activate(NULL, user_data);
}
void 
on_button_validate_dump_clicked  (GtkButton  *button, 	gpointer user_data)
{
	on_menu_utils_validate_activate(NULL, user_data);
}
// end secondary toolbar callbacks
void
on_search_dialog_find_button_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *entry_text_to_find;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	gboolean isregexp;
	gboolean is_perl_regexp;
	gboolean is_match_case;
	gboolean is_whole_word;
	gboolean is_word_start;
	gint search_flags;
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);

	entry_text_to_find =
		lookup_widget (GTK_WIDGET (button), "entry_text_to_find");
	checkbutton_regex =
		lookup_widget (GTK_WIDGET (button), "checkbutton_regex");
	radiobutton_perl_regex =
		lookup_widget (GTK_WIDGET (button), "radiobutton_perl_regex");
	checkbutton_match_case =
		lookup_widget (GTK_WIDGET (button), "checkbutton_match_case");
	checkbutton_whole_word =
		lookup_widget (GTK_WIDGET (button), "checkbutton_whole_word");
	checkbutton_word_start =
		lookup_widget (GTK_WIDGET (button), "checkbutton_word_start");

	isregexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_regex));
	is_perl_regexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (radiobutton_perl_regex));
	is_match_case =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_match_case));
	is_whole_word =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_whole_word));
	is_word_start =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_word_start));

	if (isregexp)
	{
		if (is_perl_regexp)
		{
			search_flags |= SCFIND_REGEXP;
		}
		else
		{
			search_flags |= SCFIND_POSIX;
		}
	}

	if (is_whole_word)
		search_flags |= SCFIND_WHOLEWORD;
	if (is_match_case)
		search_flags |= SCFIND_MATCHCASE;
	if (is_word_start)
		search_flags |= SCFIND_WORDSTART;
	document_search_text (doc,
			      (gchar *)
			      gtk_entry_get_text (GTK_ENTRY
						  (entry_text_to_find)),
			      search_flags);
}

void
on_dialog_replace_search_button_clicked (GtkButton * button,
					 gpointer user_data)
{
	GtkWidget *entry_text_to_find;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	gboolean isregexp;
	gboolean is_perl_regexp;
	gboolean is_match_case;
	gboolean is_whole_word;
	gboolean is_word_start;
	gint search_flags;
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);

	entry_text_to_find =
		lookup_widget (GTK_WIDGET (button), "entry_find");
	checkbutton_regex =
		lookup_widget (GTK_WIDGET (button), "checkbutton_regex");
	radiobutton_perl_regex =
		lookup_widget (GTK_WIDGET (button), "radiobutton_perl_regex");
	checkbutton_match_case =
		lookup_widget (GTK_WIDGET (button), "checkbutton_match_case");
	checkbutton_whole_word =
		lookup_widget (GTK_WIDGET (button), "checkbutton_whole_word");
	checkbutton_word_start =
		lookup_widget (GTK_WIDGET (button), "checkbutton_word_start");

	isregexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_regex));
	is_perl_regexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (radiobutton_perl_regex));
	is_match_case =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_match_case));
	is_whole_word =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_whole_word));
	is_word_start =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_word_start));

	if (isregexp)
	{
		if (is_perl_regexp)
		{
			search_flags |= SCFIND_REGEXP;
		}
		else
		{
			search_flags |= SCFIND_POSIX;
		}
	}

	if (is_whole_word)
		search_flags |= SCFIND_WHOLEWORD;
	if (is_match_case)
		search_flags |= SCFIND_MATCHCASE;
	if (is_word_start)
		search_flags |= SCFIND_WORDSTART;
	document_search_text (doc,
			      (gchar *)
			      gtk_entry_get_text (GTK_ENTRY
						  (entry_text_to_find)),
			      search_flags);
}

void
on_dialog_replace_apply_button_clicked (GtkButton * button,
					gpointer user_data)
{
	GtkWidget *entry_text_to_find;
	GtkWidget *entry_text_to_replace;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	gboolean isregexp;
	gboolean is_perl_regexp;
	gboolean is_match_case;
	gboolean is_whole_word;
	gboolean is_word_start;
	gint search_flags;
	Tcssed_window *window;
	Tcssed_doc *doc;

	gint selection_start;
	gint selection_end;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);

	selection_start = document_get_selection_start (doc);
	selection_end = document_get_selection_end (doc);

	entry_text_to_find =
		lookup_widget (GTK_WIDGET (button), "entry_find");
	entry_text_to_replace =
		lookup_widget (GTK_WIDGET (button), "entry_replace");
	checkbutton_regex =
		lookup_widget (GTK_WIDGET (button), "checkbutton_regex");
	radiobutton_perl_regex =
		lookup_widget (GTK_WIDGET (button), "radiobutton_perl_regex");
	checkbutton_match_case =
		lookup_widget (GTK_WIDGET (button), "checkbutton_match_case");
	checkbutton_whole_word =
		lookup_widget (GTK_WIDGET (button), "checkbutton_whole_word");
	checkbutton_word_start =
		lookup_widget (GTK_WIDGET (button), "checkbutton_word_start");

	if (selection_start >= selection_end)
	{
		cssed_error_message	(_("There must be some text selected\nto be replaced.\nPlease select the text to replace or repeat the search"),_("Nothing to replace") );
		return;
	}
	else
	{
		//document_new_undo_action(doc);
		document_replace_sel (doc,
				      (gchar *)
				      gtk_entry_get_text (GTK_ENTRY
							  (entry_text_to_replace)));
		//document_new_undo_action(doc);
	}
	// search next
	isregexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_regex));
	is_perl_regexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (radiobutton_perl_regex));
	is_match_case =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_match_case));
	is_whole_word =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_whole_word));
	is_word_start =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_word_start));

	if (isregexp)
	{
		if (is_perl_regexp)
		{
			search_flags |= SCFIND_REGEXP;
		}
		else
		{
			search_flags |= SCFIND_POSIX;
		}
	}
	if (is_whole_word)
		search_flags |= SCFIND_WHOLEWORD;
	if (is_match_case)
		search_flags |= SCFIND_MATCHCASE;
	if (is_word_start)
		search_flags |= SCFIND_WORDSTART;
	document_search_text (doc,
			      (gchar *)
			      gtk_entry_get_text (GTK_ENTRY
						  (entry_text_to_find)),
			      search_flags);
}
void
on_document_scanner_button_clicked(GtkButton * button,
					gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gint doclen;
	gchar* doc_buffer;
	gchar* endbuff;
	
	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);	
	
	doclen = document_get_length (tmpdoc);
	
	if( doclen == 0)
		return;
	
	doc_buffer = g_malloc0(++doclen);
	document_get_text (tmpdoc, doclen, doc_buffer);
	endbuff = g_strconcat(doc_buffer, "\n",NULL);
	document_clear_validator_arrow( tmpdoc );
	cssed_cr_parser_scan_doc_buffer (tmpdoc, endbuff, strlen(endbuff));
	g_free(doc_buffer);	
	g_free( endbuff);
}

void
on_output_list_row_activated(GtkTreeView * treeview,
					  GtkTreePath * path,
					  GtkTreeViewColumn * col,
					  gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *message;
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	gchar line[5];
	gchar number[64];
	gint i,z;

	window = CSSED_WINDOW (user_data);
	//depth = gtk_tree_path_get_depth (path);
	tmpdoc = document_get_current(window);
	
	model = gtk_tree_view_get_model (treeview);
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 0, &message, -1);
	}else{ return; }
	
	if( message[0] == '<' ){ //is markup
		for( i=0;i<strlen(message);i++){
			if( message[i] == '>')
				break;
		}
		if( i == strlen(message) ){ // bad markup gone to end of line
			g_free( message );
			return;
		}
		if( i + 3 >= strlen( message ) ){ // no room for a line number so not an error message
			g_free( message );
			return;			
		}else{ 
			i++;
		}
		if( message[i] == '[' ){
			if( i+2 >= strlen(message) ){// no room for a line number
				g_free( message );
				return;	
			}else{
				i+=1;z=0;
				memset(number, 0, 64 * sizeof(gchar));
				while( (message[i] != ']') && (z<64) && (i < strlen(message)) ){
					number[z] = message[i];
					i++;z++;
				}
				if( message[i] == ']'){ // got out in the closing brace so it's ok
					document_set_validator_arrow_at_line( tmpdoc, atoi(number) );
				}
			}
		}else{
			g_free( message );
			return;				
		}
	}	
	g_free (message);
}
/* 	clear the selector scanner store when clicked as
	buffer selection will be lost */
void
on_doc_scanner_row_activated(GtkTreeView * treeview,  GtkTreePath * path,
					  GtkTreeViewColumn * col,		  gpointer user_data)
{
	GtkListStore* current;
	Tcssed_window* window;
	
	window = CSSED_WINDOW (user_data);
	current = window->css_store;
	
	if( GTK_IS_LIST_STORE(current)){
		gtk_list_store_clear(current);
	}
	on_output_list_row_activated(treeview, path, col, user_data);	
}

void 
on_treeview_css_properties_row_activated (GtkTreeView * treeview,
					  GtkTreePath * path,
					  GtkTreeViewColumn * col,
					  gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *attribute;
	gchar *property;
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	gchar *line_to_insert;
	gchar* strdialog;
	gint depth;
	//GtkWidget *dialog;

	window = CSSED_WINDOW (user_data);
	depth = gtk_tree_path_get_depth (path);
	tmpdoc = document_get_current(window);
	
	if (depth != 3)// catch only the third col signals
	{			
		DBGMSG ("callbacks.c:on_treeview_css_properties_row_activated, depth %d ignoring", depth);
		return;
	}
	model = gtk_tree_view_get_model (treeview);
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 2, &attribute, -1);	// columns 0,1,2
		DBGMSG ("callbacks.c:on_treeview_css_properties_row_activated, attr -> %s", attribute);
	}else{
		return;
	}
	gtk_tree_path_up (path);
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 1, &property, -1);	// columns 0,1,2
		DBGMSG ("callbacks.c:on_treeview_css_properties_row_activated, property -> %s", property);
	}
	else
	{
		g_free (attribute);
		return;
	}
	// call fire dialog on document.c
	fire_dialog_from_strings_or_write_output(window,tmpdoc,attribute,property, TRUE );
	g_free (attribute);
	g_free (property);
}
void 
on_treeview_css_current_row_activated (GtkTreeView *treeview,  GtkTreePath  *path,
                                         GtkTreeViewColumn *col, gpointer     user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *attribute;
	gchar *property;
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	//gchar *line_to_insert;
	//gchar* strdialog;
	gint depth;
	GList* columns;
	GtkTreeViewColumn* first_col;
	GtkTreeViewColumn* second_col;
	GtkTreeViewColumn* third_col;
	GdkRectangle rect;
	GtkWidget* window_widget;
	GdkWindow* gdw;
	gint root_x, x, new_x;
	gint root_y, y, new_y;
	
	window = CSSED_WINDOW (user_data);
	depth = gtk_tree_path_get_depth (path);
	tmpdoc = document_get_current(window);

	columns =  gtk_tree_view_get_columns(treeview);
	
	columns = g_list_first(columns);
	if( columns != NULL ){
				first_col = columns->data;
	}
	columns = g_list_next(columns);
	if( columns != NULL ){
				second_col = columns->data;
	}	
	columns = g_list_next(columns);
	if( columns != NULL ){
				third_col = columns->data;
	}
	
	if( first_col == NULL ||
		second_col == NULL ||
		third_col == NULL )
	{
			return;			
	}
	g_list_free(columns);

	DBGMSG("callbacks.c: on_treeview_css_current_row_activated() Called from %p , Cols %p %p %p , Match third %s***",
									col, 
									first_col,
									second_col,
									third_col,
									col==third_col?"yes":"no");

	if (col != third_col)// only check third col signals
	{			
		DBGMSG ("callbacks.c: on_treeview_css_current_row_activated() tree Col %p activated, ignoring", col);
		return;
	}
	
	model = gtk_tree_view_get_model (treeview);
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 0, &property,1, &attribute, -1);	// columns 0,1,2
		gtk_tree_view_set_cursor (treeview,path,col,TRUE);
		
		// get parent col width and height to create a floating list
		gtk_tree_view_get_cell_area(treeview, path, GTK_TREE_VIEW_COLUMN(second_col), &rect);
		gdw = gtk_widget_get_parent_window(GTK_WIDGET(treeview));
		window_widget = gtk_widget_get_toplevel(GTK_WIDGET(treeview));
		gtk_widget_translate_coordinates        (GTK_WIDGET(treeview),
												 window_widget,
												 rect.x,
												 rect.y + (rect.height * 2) + 35,
												 &new_x,
												 &new_y); 
		gdk_window_get_root_origin(gdw, &root_x, &root_y);	
		x = root_x + new_x + 5;
		y = root_y + new_y;	
		create_floating_list_from_keyword(	tmpdoc, 
											property, 
											x,
											y,
											rect.width,
											100,
											model,
											gtk_tree_path_to_string(path)
										);		
		g_free(property);
		g_free(attribute);
	}else{
		DBGMSG ("callbacks.c: on_treeview_css_current_row_activated() - Nothing in the list ??");
		return;
	}	
}

void 
on_current_css_render_values_edited  (GtkCellRendererText *cell,gchar *path_string,
											gchar *new_text,gpointer user_data)
{
	Tcssed_window* window;
	Tcssed_doc* doc;
	GtkTreePath *path;
	GtkTreeIter iter;
	GtkListStore *store;
	gint start_pos;
	gint end_pos;
	gboolean valid;
	gchar* property;
	gchar* value;
	gchar* selector_contents;
	gchar* declaration;	
	
	path = gtk_tree_path_new_from_string (path_string);
	window = CSSED_WINDOW( user_data);
	store = window->css_store;	
	doc = document_get_current(window);
	
	start_pos = document_get_selection_start(doc);
	end_pos = document_get_selection_end(doc);
	
	gtk_tree_model_get_iter (GTK_TREE_MODEL(store), &iter, path);
	gtk_tree_path_free (path);
	gtk_list_store_set (store, &iter, 1, new_text, -1);
		
	DBGMSG("callbacks.c: on_current_css_render_values_edited() - Edited path %s, new text %s",path_string, new_text);

	if( start_pos == end_pos ){ // selection out
		return;
	}
	selector_contents = g_strdup(""); // empty but not NULL string must be allocated in memory
	// as will be disposed in allocated_buffer_add_string(gchar* buffer, const gchar* string)

	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
	selector_contents = allocated_buffer_add_string(selector_contents, "\n\t");
	while( valid ){
		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter,
							0, &property, 
							1, &value, 	-1);

		DBGMSG("callbacks.c: on_current_css_render_values_edited() - property : %s - value : %s",property, value);
				
		if( (property != NULL) || (value != NULL) ){ // avoid NULL values
			declaration = g_strdup_printf("%s:%s;",property,value);
			selector_contents = allocated_buffer_add_string(selector_contents, declaration);
			g_free(declaration);
			g_free(property);
			g_free(value);
		}
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);	
		if( valid ){
			selector_contents = allocated_buffer_add_string(selector_contents, "\n\t");
		}else{
			selector_contents = allocated_buffer_add_string(selector_contents, "\n");
		}			
	}	
	document_replace_sel(doc,selector_contents);
	g_free( selector_contents );
	on_menu_scan_selector_activate (NULL, window);
}
/* /////////////// EDITOR CALLBACKS TESTING //////////////////// */
/* This will fire some user list in the future, here for listening to the 
 function keys and others, not used yet */
gboolean
on_editor_key_press_event(GtkWidget *widget,    GdkEventKey *event,
                                            gpointer user_data)
{
	DBGMSG("callbacks.c: on_editor_key_press_event() - editor key pressed");
	return FALSE;
}

/* this will erase all data in the current css list when the user clicks again 
	on the editor widget, or changes something on it. This way, we are sure that
	the current selector list is in sync with the selected text in the editor */
void
on_editor_grab_focus           (GtkWidget *widget,
                                gpointer user_data)
{
	Tcssed_window* window;
	GtkWidget* treeview;
	GtkTreeModel* list_store;
	GtkTreeIter iter;
	gboolean valid;
	
	window = CSSED_WINDOW(user_data);	
	treeview =	window->tree_view_css_current;
	
	DBGMSG("callbacks.c: on_editor_grab_focus() : editor %p grab focus",widget);

	list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	valid = gtk_tree_model_get_iter_first (list_store, &iter);
	
	if( valid ){ // clean all data	
		DBGMSG("callbacks.c: on_editor_grab_focus() - There was data in the css current properties list, cleaning ...");
		gtk_list_store_clear(GTK_LIST_STORE(list_store));
	}
}
	
/* /////////////////// EDITOR CALLBACKS END ////////////////// */
void 
on_menu_autocompletion_activate	
		(GtkMenuItem	  *menuitem,   		gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;
	gchar* list;
	
	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	list = window->keywords_list;

	document_show_autocompletion_list(doc,0,list);	
}

void
popmenu_autocomp_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
{
	Tcssed_popmenu_data* data;
	Tcssed_window* window;
	Tcssed_doc* doc;
	gchar* attribute;
	gchar* property; // previous_string
	gboolean add_property_string;	
	
	data = (Tcssed_popmenu_data*) user_data;	
	window = CSSED_WINDOW(data->main_window);
	doc = (Tcssed_doc*) data->document;

	DBGMSG("callbacks.c: popmenu_autocomp_activate_cb - %s - %s",(gchar*) data->attribute,(gchar*) data->property);

	attribute = g_strdup( (gchar*) data->attribute);
	property = g_strdup( (gchar*) data->property); 
	add_property_string = data->add_property_string;
	// Memory is disposed in parent's "unmap" event ( see below ). This menu is only for one use.
	fire_dialog_from_strings_or_write_output(window,doc,attribute,property,add_property_string);
	g_free(attribute);
	g_free(property);
}

/* When the pop menu is created some memory is dynamically allocated and must
   be disposed. This callback will track the popmenu unmap event and clear all
   memory allocated. The pointers to the data allocated are in a doubly
   linked list passed as user_data  */
void
popmenu_autocomp_event_after_cb(GtkWidget *widget,
                                GdkEvent *event,
                                gpointer user_data)
{
	GList* list;
	Tcssed_popmenu_data* data;
	
	if ( event->type == GDK_UNMAP) {
		list = g_list_first((GList*) user_data);

		DBGMSG("callbacks.c: popmenu_autocomp_event_after_cb() - After menu unmap event let's dispose allocated memory");
	
		if( list != NULL ){
			while( list != NULL ){
				data = (Tcssed_popmenu_data*) list->data;
				DBGMSG("callbacks.c: popmenu_autocomp_event_after_cb() - Cleaning element %p - containig %s %s",
							data, data->attribute,data->property);
				g_free(data->attribute);
				g_free(data->property);
				g_free(data);
				list = g_list_next(list);
			}
			g_list_free (list);
		}
#ifdef DEBUG	
		else
		{
			DBGMSG("callbacks.c: popmenu_autocomp_event_after_cb() - List is empty or NULL");		
		}
#endif	
		g_list_free(user_data);	
		
		if( GTK_IS_WIDGET(widget) ){// ensure the menu is destroyed
			gtk_widget_destroy(widget);
			DBGMSG("callbacks.c: popmenu_autocomp_event_after_cb() - Menu destroyed");
		}
	}
}
/* common to all file selector dialogs */
void file_selector_close (GtkDialog       *dialog,
                          gpointer         user_data)
{
	g_free(user_data);
}
									
void
file_selector_cancel_button_clicked (GtkButton       *button,  
									gpointer         user_data)
{
	GtkWidget* dialog;
	
	dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
	if(GTK_WIDGET_TOPLEVEL (dialog)){ //ensure it's the toplevel
		gtk_widget_destroy(dialog);
	}
	g_free(user_data);
}
/* ***************************** HELP BUTTONS ******************************* */
/*    ---------------------------   TO DO ----------------------------------  */
void
on_color_selection_help_button_clicked (GtkButton * button, gpointer user_data)
{
}

void
on_dialog_search_help_button_clicked (GtkButton * button, gpointer user_data)
{
}


void
on_dialog_replace_help_button_clicked (GtkButton * button, gpointer user_data)
{
}
